'''ola.py'''

"""
The audiotsm.ola module implements the OLA (Overlap-Add) time-scale
modification procedure.
"""

from audiotsm2.base import AnalysisSynthesisTSM, Converter
from audiotsm2.utils.windows import hanning


class OLAConverter(Converter):
    """
    A Converter implementing the OLA (Overlap-Add) time-scale modification
    procedure.
    """
    def convert_frame(self, analysis_frame):
        return analysis_frame


def ola(channels, speed=1., frame_length=256, analysis_hop=None,
        synthesis_hop=None):
    """
    Returns a audiotsm.base.tsm.TSM object implementing the OLA
    (Overlap-Add) time-scale modification procedure.

    In most cases, you should not need to set the frame_length, the
    analysis_hop or the synthesis_hop. If you want to fine tune these
    parameters, you can check the documentation of the
    audiotsm.base.analysis_synthesis.AnalysisSynthesisTSM class to
    see what they represent.
    """
    if(synthesis_hop is None):
        synthesis_hop = frame_length // 2

    if(analysis_hop is None):
        analysis_hop = int(synthesis_hop * speed)

    analysis_window = None
    synthesis_window = hanning(frame_length)

    converter = OLAConverter()

    return AnalysisSynthesisTSM(
        converter, channels, frame_length, analysis_hop, synthesis_hop,
        analysis_window, synthesis_window)



'''io/stream.py'''

import numpy as np
from sounddevice import OutputStream

class StreamWriter():
    """
    A audiotsm2.io.base.Writer allowing to play the output of a
    audiotsm2.base.tsm.TSM object directly.
    """
    def __init__(self, channels, samplerate, **attrs):
        self._channels = channels

        self._stream = OutputStream(samplerate=samplerate, channels=channels, **attrs)
        self._stream.start()

    @property
    def channels(self):
        return self._channels

    def write(self, buffer):
        if buffer.shape[0] != self.channels:
            raise ValueError("the buffer should have the same number of "
                             "channels as WavWriter")

        self._stream.write(np.ascontiguousarray(buffer.T))

        return buffer.shape[1]

    def stop(self):
        self._stream.stop()

    def __enter__(self):
        return self

    def __exit__(self, _1, _2, _3):
        self.stop()



#!/usr/bin/env python3

import os
import re
import sys
from setuptools import setup, find_packages

def find_version():
    version_filename = os.path.abspath('audiotsm2/__init__.py')
    with open(version_filename) as fileobj:
        version_content = fileobj.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
        version_content, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    os.system('rm -rf build')
    os.system('rm -rf dist')
    os.system('python3 setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()


with open('README.md', 'r') as f:
    long_description = f.read()


setup(
    name='audiotsm2',
    version=find_version(),
    description='A real-time audio time-scale modification library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    url='https://github.com/WyattBlue/audiotsm2',
    author='WyattBlue',
    author_email='wyattbluesandbox@gmail.com',

    packages=find_packages(),

    install_requires=[
        'numpy',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)
